//+------------------------------------------------------------------+
//|   Steve Hopwood's London session breakout auto trading robot.mq4 |
//|                                                                  |
//|                                                             |
//+------------------------------------------------------------------+

/*+------------------------------------------------------------------+
 *
 * Version history:
 *
 * V4.0: [Squalou]:
 *     - Added TRAILING STOP capability: 
 *       new inputs: 
 *          TrailingStopPips(0): pips to trail the StopLoss; when 0 no trailing is applied (fixed SL as before).
 *          TrailingStopStep(1): trailing SL will jump by this amount instead of trailing at every tick;
 *          (helps limit the number of sent orders, and therefore order rejections).
 *
 *     - Added profit lock-in at fixed TP:
 *       when "BreakEvenPips" is >0, SL is moved to BE+"BreakEvenProfitInPips" when price has reached BE+"BreakEvenPips";
 *       Works independently from "AllowHalfClose" option (but uses the same BreakEvenProfitInPips value);
 *       Will also be trailed if TrailingStop is selected;
 *       (inspired from Steve's MPTM EA...)
 *
 *     - Added MaxRisk input: if >0, then order size is the min of Lot input and calculated max Lot based on MaxRisk and Stop-Loss;
 *
 *     - All open/pending trades will be closed when the EA is unloaded;
 *
 *     - when MagicNumber is 0, a unique MagicNumber is created by the EA, MagicNumbers will be different for each pair/TF;
 *       This helps running the EA on different pairs/TF without having to 
 *
 *     - fix: in V3.0, objPrefix default value was not set to "LB-" as it should have been, leading to error 130 "invalid stops";
 *     - added Alerts when Indicator SETTINGS GlobalVariables cannot be found; EA will stop trading in this case.
 *
 * V3.0: [Squalou]:
 *       ========== ALLOWS BACKTESTING THE EA ==========
 *     - the EA now computes the Entry/TP/SL LEVELS by itself, and only imports Indicator SETTINGS
 *       from the GlobalVariables.
 *     - EA now starts trading at the END of the bar forming at the Box EndTime, as expected by the Strategy:
 *       on M15, with EndTime=07:00, EA will start trading at 07:15, not 07:01.
 *     - fixed recursive alarms playing when price goes out of the "green fields";
 *       Only valid pending STOP orders are now placed based on market price;
 *     - Fixed OCO function that was repeatedly trying to delete wrong orders;
 *     - fixed count of losses for the day: when EA was used on multiple pairs/charts,
 *       ALL running EAs would "stop trading for the day" when the FIRST one reached MaxLossesAllowedPerPair...
 *     - fixed Half-close not functionning on SELL side;
 *
 * V2.0: 
 *     - designed to work with "London BreakoutV8.1.mq4" indicator "exported" GobalVariables:
 *       grabs "MaxBoxSizeInPips", "StartTime", "EndTime", and "SessionEndTime"
 *
 * V1: original version from SteveHopwood at forexfactory.com
 *     - designed to work with "London BreakoutV8.mq4" indicator "exported" GobalVariables:
 *
 *+------------------------------------------------------------------+
 */

#property copyright "Open source -- by SteveHopwood & Squalou"

#property show_inputs
#include <WinUser32.mqh>
#include <stdlib.mqh>


#define  EA_VERSION    "V4.0"

//+------------------------------------------------------------------+

extern string     _bs="----Basic stuff----";
extern double     Lot  = 22;
extern double     MaxRisk = 0; // if 0 then use fixed Lots above
extern int        MagicNumber=0; // if 0 then will autocreate a unique MagicNumber
extern string     TradeComment="----breakout EA";
extern string     _mta1="----MaxTradesAllowedForTheDay = 0";
extern string     _mta2="----means no limit on trades";
extern int        MaxTradesAllowedForTheDay=0;
extern string     _mla1="----MaxLossesAllowedPerPair = 0";
extern string     _mla2="----means no limit on losses";
extern int        MaxLossesAllowedPerPair=2;
extern string     objPrefix = "LB-";  // Prefixes of London Breakout Indicator objects;
extern string     _tm="----Trade management----";
extern bool       UseSelectedManagementFunctions=true;
extern string     _hcf="----Half-close function";
extern bool       AllowHalfClose=true;
extern int        BreakEvenPips=1000; // when >0, will move SL to BE+BreakEvenProfitInPips when price has reached BE+BreakEvenPips
extern int        BreakEvenProfitInPips=500;  // lock-in this amount of pips when HalfClose is hit;
extern int        TrailingStopPips = 1500;// Pips to trail the StopLoss; if 0 then no trailing stop
extern int        TrailingStopStep = 100;// trailing SL jumping step in pips
extern string     _oco1="----OCO will delete remaining pending";
extern string     _oco2="----when its partner fills";
extern bool       OneCancelsTheOther=false;
/*
extern string     _mart1="----martingale multipliers sequence:";
extern string     _mart2="----not used when first multiplier is 0:";
extern string     martingale_sequence="1,2,5,13,34,89,233,610,1597,4181";
*/
extern string     _misc="----Misc:";
extern int        DisplayGapSize=100;

//+------------------------------------------------------------------+

#define  NL    "\n"
#define  Status_initializing "????? I N I T I A L I Z I N G ?????"
#define  Status_initialized  "..... I N I T I A L I Z E D ....."
#define  Status_trading "+++++ T R A D I N G +++++"
#define  Status_not_trading "!!!!! N O T   T R A D I N G (Box is too large) !!!!!"
#define  Status_finished "----- F I N I S H E D   F O R   T O D A Y -----"

//Entry, stop and tp
double  LongPendingEntryPrice;
double  LongTP;
double  LongSL;
double  ShortPendingEntryPrice;
double  ShortTP;
double  ShortSL;

// Variables imported from IndicatorV8.1:
string  StartTime;         // time for start of price establishment window
string  EndTime;           // time for end of price establishment window
string  SessionEndTime;    // end of daily session; tomorrow is another day!
int     MaxBoxSizeInPips;  // box size limit to show Red "NO TRADE" box
double  EntryFactor;
double  TPFactor;
double  LevelsResizeFactor;
int     MinExtentInPips; // actual box extent min limit for computing trade levels;
int     MaxExtentInPips; // actual box extent max limit for computing trade levels; 0 will default to MaxBoxSizeInPips;

int iStartTime,iEndTime,iSessionEndTime;

int _iStartTime,_iEndTime,_iSessionEndTime;
int     _MaxBoxSizeInPips;  // box size limit to show Red "NO TRADE" box
double  _EntryFactor;
double  _TPFactor;
double  _LevelsResizeFactor;
int     _MinExtentInPips; // actual box extent min limit for computing trade levels;
int     _MaxExtentInPips; // actual box extent max limit for computing trade levels; 0 will default to MaxBoxSizeInPips;

datetime tBoxStart;
datetime tBoxEnd;
datetime tSessionStart;
datetime tSessionEnd;

double vSellStop;
double vBuyStop;

int StopLoss;

int StartShift;
int EndShift;
double boxHigh;
double boxLow;
double boxExtent;
double boxMedianPrice;

string symbol;


double     LondonOpens;
double     LondonCloses;

//Open trades\trade direction.
string            TradingStatus;

//Trading hours
bool              ClosedForFriday;
bool              ClosedForSaturday;

//Hi-lo variables
//double            bhighest, blowest;//Box highest and lowest


//Open trades\trade direction.
bool              TradeOpen;
int               TicketNo, LongTicketNo, ShortTicketNo;

//Aggregates
int               open, won, lost, total;

//Misc stuff
string            comment, Gap;
bool              RobotDisabled;
string            DisabledMessage;

// market info
double pip;
int digits;
double minlot;

//Risk Management
int    maxSL;

string op_str[5] = { "BUY","SELL","BUYLIMIT","SELLLIMIT","BUYSTOP","SELLSTOP"};

//+------------------------------------------------------------------+
void DisplayUserFeedback()
//+------------------------------------------------------------------+
{

   comment = StringConcatenate(Gap, WindowExpertName(), " ", EA_VERSION, NL);
   comment = StringConcatenate(comment, Gap, "Lot size = ", Lot, ", MaxRisk = ", MaxRisk, "%", NL);
   comment = StringConcatenate(comment, Gap, "Magic number = ", MagicNumber, ", objPrefix = ", objPrefix, NL);
   comment = StringConcatenate(comment, Gap, "Trade comment = ", TradeComment, NL);
   comment = StringConcatenate(comment, Gap, "Local time: ", TimeToStr(TimeLocal()), " Broker time: ", TimeToStr(TimeCurrent()), NL);
   comment = StringConcatenate(comment, Gap, "Long Entry: ", DoubleToStr(LongPendingEntryPrice, Digits), ", TP: ", DoubleToStr(LongTP, Digits), ", SL: ", DoubleToStr(LongSL, Digits), NL);
   comment = StringConcatenate(comment, Gap, "Short Entry: ", DoubleToStr(ShortPendingEntryPrice, Digits), ", TP: ", DoubleToStr(ShortTP, Digits), ", SL: ", DoubleToStr(ShortSL, Digits), NL);
   comment = StringConcatenate(comment, Gap, "Trading Hours: ", DoubleToStr(LondonOpens/100,2), " - ", DoubleToStr(LondonCloses/100,2), NL);

   if (UseSelectedManagementFunctions)
   {
      comment = StringConcatenate(comment, Gap, "Using management functions:",NL);
      if (AllowHalfClose) comment = StringConcatenate(comment,     Gap, "     Using half - close",NL);
      else                comment = StringConcatenate(comment,     Gap, "     NOT Using half - close",NL);
      comment =                     StringConcatenate(comment,     Gap, "     lock-in ",BreakEvenProfitInPips," pips when reached BE+",BreakEvenPips," pips profit", NL);
      if (OneCancelsTheOther) comment = StringConcatenate(comment, Gap, "     OCO ON:   Closing parter pending trade when one fills", NL);
      else                    comment = StringConcatenate(comment, Gap, "     OCO OFF: NOT Closing parter pending trade when one fills", NL);
   }//if (UseSelectedManagementFunctions)
   
   comment = StringConcatenate(comment, Gap, EA_VERSION, " " , TradingStatus, NL);

   Comment(comment);
}//void DisplayUserFeedback()


//+------------------------------------------------------------------+
int init()
//+------------------------------------------------------------------+
{

  TradingStatus = Status_initializing;

  getpip(); // get pip multiplier and digit

  symbol = Symbol(); // used throughout all functions

  //Create a MagicNumber if it is 0
  if (MagicNumber == 0) {
    MagicNumber = create_MagicNumber("");
  }
    
  if (!IsDemo() )
  {
    if (MagicNumber == 8682371)
    {
      MessageBox("You cannot run this robot until changed the MagicNumber input from the default of 8682371." + NL 
                + "Please reload the robot with a new MagicNumber input.");
      DisabledMessage = "You need to change the MagicNumber input";
      RobotDisabled = true;
      _Alert (DisabledMessage);
      Comment("The robot has been suspended. Reason: ", DisabledMessage);
      return;
    }
  
    if (MagicNumber < 10000) { //Insufficient length
      MessageBox("Your MagicNumber input needs to be at least 5 Digits long, your MagicNumber is " + MagicNumber + NL 
                 + "Please reload the robot with the MagicNumber input containing at least 5 Digits - the more the better.");
      DisabledMessage = "You need to make the MagicNumber input at 5 Digits long - more is better";
      _Alert (DisabledMessage);
      Comment("The robot has been suspended. Reason: ", DisabledMessage);
      RobotDisabled = true;
      return;
    }
  }

  if (Lot < minlot) {
    DisabledMessage = "Lot must be >= "+minlot+NL;
    _Alert (DisabledMessage);
    Comment("The robot has been suspended. Reason: ", DisabledMessage);
    RobotDisabled = true;
    return;
  }

  if (AllowHalfClose && Lot/2 < minlot) {
    DisabledMessage = "Lot must be >= "+2*minlot+ " to allow HalfClose option;"+NL;
    _Alert (DisabledMessage);
    Comment("The robot has been suspended. Reason: ", DisabledMessage);
    RobotDisabled = true;
    return;
  }


  //Check MaxRisk value
  if (MaxRisk < 0 || MaxRisk > 100)
  {
    MessageBox("INVALID MaxRisk value " + MaxRisk + NL 
               + "Please reload the robot with valid MaxRisk.");
    DisabledMessage = "INVALID MaxRisk value " + MaxRisk;
    RobotDisabled = true;
    _Alert (DisabledMessage);
    Comment("The robot has been suspended. Reason: ", DisabledMessage);
    return;
  }


  Gap=""; for (int cc=0; cc< DisplayGapSize; cc++) Gap = StringConcatenate(Gap, " ");

  if (!import_LB_Indi_SETTINGS(objPrefix,symbol)) {
    DisabledMessage = "Some Indicator SETTINGS are missing;\n    Please verify objPrefix in the Indicator and the EA;\n    (objPrefix is currently set to "+objPrefix+" in the EA)";
    RobotDisabled = true;
    _Alert (DisabledMessage);
    Comment("The robot has been suspended. Reason: ", DisabledMessage);
    return;
  }

  update_local_LB_Indi_SETTINGS();
  // compute LEVELS values:
  compute_LB_Indi_LEVELS(Time[0]);

  TradingStatus = Status_initialized;
  DisplayUserFeedback();
  return(0);
}
  
//+------------------------------------------------------------------+
int deinit()
//+------------------------------------------------------------------+
{

  Comment("");

  CloseAllTrades();

  return(0);

}
  
//+------------------------------------------------------------------+
int start()
//+------------------------------------------------------------------+
{
  double servers_min_stop = MarketInfo(symbol, MODE_STOPLEVEL) * MarketInfo(symbol, MODE_POINT);

  if (RobotDisabled)
  {
    Comment("The robot has been suspended. Reason: ", DisabledMessage);
    return;
  }


  /* Import SETTINGS input values from GlobalVariables:
   * available since "London BreakoutV8.mq4" indicator;
   */
  if (!import_LB_Indi_SETTINGS(objPrefix,symbol)) {
    _Alert("");
    DisabledMessage = "Some Indicator SETTINGS are missing;\n    Please verify objPrefix in the Indicator and the EA;\n    (objPrefix is currently set to "+objPrefix+" in the EA)";
    RobotDisabled = true;
    _Alert (DisabledMessage);
    return;
  }

  // if Indicator SETTINGS have been changed, then CLOSE ALL TRADES (pending AND open trades);
  // EA will reopen pending trades based on the new settings;
  if (update_local_LB_Indi_SETTINGS()) {
    CloseAllTrades();
  }

  // compute LEVELS values:
  compute_LB_Indi_LEVELS(Time[0]);

  // setup session open/close times
  LondonOpens  = roundup_hhmm_time_to_TF (time_string_to_int(EndTime), Period());
  LondonCloses = time_string_to_int(SessionEndTime);

  if (!IsTradeAllowed())
  {
    Comment("Trading disabled by the criminal pillock. Please wait for him to get out of bed.");
    return;
  }//if (!IsTradeAllowed)



  //Trading hours
  bool TradeHour = false;
  int now = TimeHour(Time[0])*100 + TimeMinute(Time[0]);
  if (LondonOpens <= LondonCloses && (now >= LondonOpens && now <  LondonCloses)) TradeHour = true;
  if (LondonOpens >  LondonCloses && (now <  LondonOpens || now >= LondonCloses)) TradeHour = true;
  if (!TradeHour)
  {
     if (TradingStatus != Status_finished) {
       _Alert (Status_finished+" Session End at "+DoubleToStr(LondonCloses/100,2));
       CloseAllTrades();
     }
     TradingStatus = Status_finished;
     DisplayUserFeedback();
     return;
  }//if (!TradeHour)

  // if box is too large, don't trade today!
  GetTradingStatus();
  if (TradingStatus == Status_not_trading)
  {
    DisplayUserFeedback();
    return;
  }


  GetAggregatePosition();            

  if (OrdersTotal() > 0 && UseSelectedManagementFunctions) {
    //-----------------------
      ManageOpenTrades();
    //-----------------------
  }

  //Max trades filter
  if (MaxTradesAllowedForTheDay > 0)// 0 means unlimited trades
  {                
    if (total >= MaxTradesAllowedForTheDay) 
    {
      if (TradingStatus != Status_finished) {
        _Alert (Status_finished+"Reached Max Trades at "+TimeToStr(TimeCurrent(),TIME_SECONDS));
        CloseAllTrades();
      }
      TradingStatus = Status_finished;
      DisplayUserFeedback();
      return;
   }
  }

  //Max loss filter
  if (MaxLossesAllowedPerPair > 0)
  {
    int lost = LossesCount();
    if (lost >= MaxLossesAllowedPerPair) 
    {
      if (TradingStatus != Status_finished) {
        _Alert (Status_finished+"Reached MAX LOSSES at "+TimeToStr(TimeCurrent(),TIME_SECONDS));
        CloseAllTrades();
      }
      TradingStatus = Status_finished;
      DisplayUserFeedback();
      return;
    }
  }


  if (IsTradeAllowed())
  {                    
    TradingStatus = Status_trading;

    RefreshRates();

    //adjust Lot based on MaxRisk and SL value
    double lotsToTrade = Lot;
    if (MaxRisk > 0) lotsToTrade = MathMin(Lot,GetMaxLot(StopLoss,MaxRisk));
    //If market is in the box, then check to see if pending trades need sending, and send them if so
    //Long trade
    if (Ask <= LongPendingEntryPrice-servers_min_stop && !DoesTradeExist(OP_BUYSTOP) && !DoesTradeExist(OP_BUY) && !DoesTradeExist(OP_SELL))
      SendPendingOrder(OP_BUYSTOP, lotsToTrade, LongPendingEntryPrice, LongSL, LongTP);
    //Short
    if (Bid >= ShortPendingEntryPrice+servers_min_stop && !DoesTradeExist(OP_SELLSTOP) && !DoesTradeExist(OP_SELL) && !DoesTradeExist(OP_BUY))
      SendPendingOrder(OP_SELLSTOP, lotsToTrade, ShortPendingEntryPrice, ShortSL, ShortTP);
  }
        
  
  DisplayUserFeedback();
  return(0);

}//End start()

//+------------------------------------------------------------------+
double GetMaxLot(int SL, double Risk)
//+------------------------------------------------------------------+
{
  double lotsize,tickvalue,pipvaluefor1lot,SLfor1lotfor1percent,maxLot;
  lotsize   = MarketInfo(Symbol(),MODE_LOTSIZE);
  tickvalue = MarketInfo(Symbol(),MODE_TICKVALUE);
  pipvaluefor1lot = lotsize*tickvalue*pip;
  maxLot    = NormalizeDouble(AccountFreeMargin()*Risk/100/pipvaluefor1lot/SL,2);
  return(maxLot);
}

//+------------------------------------------------------------------+
bool DoesTradeExist(int type)
//+------------------------------------------------------------------+
{
   //Searches for open trades
   for (int cc = OrdersTotal() - 1; cc >= 0; cc--)
   {      
      if (!OrderSelect(cc, SELECT_BY_POS) ) continue;
      if (OrderMagicNumber() == MagicNumber && OrderSymbol() == symbol && OrderCloseTime() == 0) 
      {
         if (OrderType() == type) return(true);
      }
   }
   
   return(false);
   
} //DoesTradeExist


//+------------------------------------------------------------------+
void SendPendingOrder(int type, double lots, double price, double SL, double TP) //OP_BUYSTOP or OP_SELLSTOP
//+------------------------------------------------------------------+
{
  //Martingale code
  /*if (UseMartingale && lost > 0)
  {

  }
  */

  //Buy stop trade
  _Alert("Sending "+op_str[type]+" at "+price+" "+lots+" lots");

  _OrderSend(symbol, type, lots, price, 0, SL, TP, TradeComment, MagicNumber, 0,Blue);

}//void SendPendingOrder()


//+------------------------------------------------------------------+
void GetAggregatePosition()
//+------------------------------------------------------------------+
{
   //returns the total trades for the pair - open, won, lost, also the ticket nos for open trades
   
   open = 0;
   won = 0;
   lost = 0;
   total = 0;
   LongTicketNo = 0;
   ShortTicketNo = 0;
   
   
   //Open trades
   if (OrdersTotal() > 0)
   {
      for (int cc = OrdersTotal() - 1; cc >= 0; cc--)
      {      
         if (!OrderSelect(cc, SELECT_BY_POS, MODE_TRADES) ) continue;
         if (OrderMagicNumber() == MagicNumber && OrderSymbol() == symbol && OrderCloseTime() == 0)
         {
            open++;
            if (OrderType() == OP_BUY  || OrderType() == OP_BUYSTOP)  LongTicketNo  = OrderTicket();
            if (OrderType() == OP_SELL || OrderType() == OP_SELLSTOP) ShortTicketNo = OrderTicket();
         }
      }
   }
   
   //History tab
   for (cc = OrdersHistoryTotal() - 1; cc >= 0; cc--)
   {      
      OrderSelect(cc, SELECT_BY_POS, MODE_HISTORY);
      if (OrderMagicNumber() == MagicNumber && OrderSymbol() == symbol)       
      {
         if (OrderProfit() < 0) lost++;
         if (OrderProfit() > 0) won++;         
      }
   }
   
   total = open + lost + won;

}//void GetAggregatePosition()

//+------------------------------------------------------------------+
int LossesCount()
//+------------------------------------------------------------------+
{
    int Cnt=0;
    for(int i = OrdersHistoryTotal()-1; i>=0; i--)
    {
      OrderSelect(i,SELECT_BY_POS,MODE_HISTORY);
      if (OrderMagicNumber() == MagicNumber && OrderSymbol() == symbol) {
        if (TimeDayOfWeek(OrderCloseTime()) == TimeDayOfWeek(Time[0]) ) {
           if(OrderProfit()<0) Cnt++;
        }
      }
    }
    return(Cnt);
}//LossesCount()

//+------------------------------------------------------------------+
void GetTradingStatus()
//+------------------------------------------------------------------+
{
   //The initial trade stops are the extent of the box
   
  if (MaxBoxSizeInPips > 0 && boxExtent > MaxBoxSizeInPips * pip) TradingStatus = Status_not_trading;

}//End void GetTradingStatus()

//+------------------------------------------------------------------+
void ManageOpenTrades()
//+------------------------------------------------------------------+
{
  //Called if there are any open trades
  for (int cc = OrdersTotal() - 1; cc >=0; cc--)
  {
    if (!OrderSelect(cc, SELECT_BY_POS) ) continue;

    if (OrderMagicNumber() ==  MagicNumber && OrderSymbol() == symbol && OrderCloseTime() == 0)
    {
      TicketNo = OrderTicket();

      // Half close at half-TP ?
      if (AllowHalfClose) {
        if (OrderCloseTime() == 0 && OrderProfit() > 0 && OrderLots() == Lot) {
          TicketNo = ShouldHalfTradeClose(TicketNo); // returns new ticket assigned to the half-closed trade;
        }
      }

      // Move to BreakEven+some_pips after some profit ?
      if (BreakEvenPips > 0) {
        BreakEvenStopLoss(BreakEvenPips/*profit_pips_target*/, BreakEvenProfitInPips/*pips_to_lock*/);
      }
   
      // OCO ?
      if (OneCancelsTheOther) {
        OCO();
      }

      // Trailing stop ?
      if (TrailingStopPips > 0) {
        Trail_order(TrailingStopPips,TrailingStopStep);
      }

    }
  }

}//End void ManageOpenTrades()

//+------------------------------------------------------------------+
int ShouldHalfTradeClose(int TicketNo)
//+------------------------------------------------------------------+
{
  //Only called if selected trade is in profit;
  //Closes half the trade at profit of TP / 2;
   
  bool result;
  double SL;
  double HcPrice = NormalizeDouble((OrderTakeProfit() + OrderOpenPrice()) / 2, Digits);//Half close price
   
  //Long
  if (OrderType() == OP_BUY && Bid > HcPrice) {
    _Alert(" #"+OrderTicket()+" half-close Buy at "+Bid);
    result = _OrderClose(OrderTicket(),OrderLots() / 2, Bid,30,Red);
    if (result==false) return (TicketNo);
    //Get new ticket number
    Sleep(100);
    result = DoesTradeExist(OP_BUY);//Re-selects the half-closed trade
    if (result==false) {
      _Alert(" #"+OrderTicket()+" DoesTradeExist(OP_BUY) FAILED ");
      return (TicketNo);
    }
    TicketNo = OrderTicket();//get new ticket, will be returned to caller;
    if (OrderStopLoss() < OrderOpenPrice())
    {
      SL = NormalizeDouble(OrderOpenPrice() + (BreakEvenProfitInPips * pip), Digits);
      _Alert(" #"+OrderTicket()+" moving to BE+"+BreakEvenProfitInPips+"pips");
      result = _OrderModify(OrderTicket(), OrderOpenPrice(), SL, OrderTakeProfit(), OrderExpiration(), Green);
    }
    return (TicketNo);
  }
   
  //Short
  if (OrderType() == OP_SELL && Ask < HcPrice) {
    _Alert(" #"+OrderTicket()+ " half-close Sell at "+ Ask);
    result = _OrderClose(OrderTicket(),OrderLots() / 2, Ask,30,Red);
    if (result==false) return (TicketNo);
    //Get new ticket number
    Sleep(100);
    result = DoesTradeExist(OP_SELL);//Re-selects the half-closed trade
    if (result==false) {
      _Alert(" #"+OrderTicket()+ " DoesTradeExist(OP_SELL) FAILED ");
      return (TicketNo);
    }
    TicketNo = OrderTicket();//get new ticket, will be returned to caller;
    if (OrderStopLoss() > OrderOpenPrice() || OrderStopLoss()==0)
    {
      SL = NormalizeDouble(OrderOpenPrice() - (BreakEvenProfitInPips * pip), Digits);
      _Alert(" #"+OrderTicket()+ " moving to BE+"+BreakEvenProfitInPips+"pips");
      result = _OrderModify(OrderTicket(), OrderOpenPrice(), SL, OrderTakeProfit(), OrderExpiration(), Green);
    }
    return (TicketNo);
  }
   
  return (TicketNo);
}//ShouldHalfTradeClose()

//+------------------------------------------------------------------+
void BreakEvenStopLoss(int profit_pips_target, int pips_to_lock)
//+------------------------------------------------------------------+
{
  //Lock-in some profit after a minimum profit target is reached:
  //(move SL to BE+pips_to_lock)
  double SL;
   
  //Long
  if (OrderType() == OP_BUY && Bid >= OrderOpenPrice()+profit_pips_target*pip && OrderStopLoss() < OrderOpenPrice()) {
    SL = NormalizeDouble(OrderOpenPrice() + (pips_to_lock*pip), Digits);
    _Alert(" #"+OrderTicket()+" moving to BE+"+pips_to_lock+"pips");
    _OrderModify(OrderTicket(), OrderOpenPrice(), SL, OrderTakeProfit(), OrderExpiration(), Green);
  }
   
  //Short
  if (OrderType() == OP_SELL && Ask <= OrderOpenPrice()-profit_pips_target*pip && (OrderStopLoss() > OrderOpenPrice() || OrderStopLoss()==0)) {
    SL = NormalizeDouble(OrderOpenPrice() - (pips_to_lock*pip), Digits);
    _Alert(" #"+OrderTicket()+" moving to BE+"+pips_to_lock+"pips");
    _OrderModify(OrderTicket(), OrderOpenPrice(), SL, OrderTakeProfit(), OrderExpiration(), Green);
  }
   
}//BreakEvenStopLoss()

//+------------------------------------------------------------------+
void OCO() //One Cancels The Other
//+------------------------------------------------------------------+
{
  //Deletes a pending trade when its partner has filled
  if (OrderType()==OP_BUY) {
    if (DoesTradeExist(OP_SELLSTOP)) {
      _Alert(" #"+OrderTicket()+ " OCO(): deleting pending SELLSTOP");
      _OrderDelete(OrderTicket(),CLR_NONE);
    }
  }

  if (OrderType()==OP_SELL) {
    if (DoesTradeExist(OP_BUYSTOP)) {
      _Alert(" #"+OrderTicket()+ " OCO(): deleting pending BUYSTOP");
      _OrderDelete(OrderTicket(),CLR_NONE);
    }
  }
}// OCO()


//+------------------------------------------------------------------+
void Trail_order(int _TrailingStopPips, int _TrailingStopStep)
//+------------------------------------------------------------------+
{
  if (_TrailingStopPips<=0) return;
  if (_TrailingStopStep>_TrailingStopPips) return;
  if (MathAbs(OrderStopLoss()-OrderClosePrice()) > (_TrailingStopPips+_TrailingStopStep)*pip) {										    
    if (OrderType() == OP_BUY)  _OrderModify(OrderTicket(),OrderOpenPrice(),OrderClosePrice()-_TrailingStopPips*pip,OrderTakeProfit(),OrderExpiration(),Purple);
    if (OrderType() == OP_SELL) _OrderModify(OrderTicket(),OrderOpenPrice(),OrderClosePrice()+_TrailingStopPips*pip,OrderTakeProfit(),OrderExpiration(),Purple);
  }
}

//+------------------------------------------------------------------+
void CloseAllTrades()
//+------------------------------------------------------------------+
{
  bool result;
  int err;

  _Alert(" CLOSING ALL ORDERS NOW !!!");

  for (int cc = OrdersTotal() - 1; cc >=0; cc--)
  {
    if (!OrderSelect(cc, SELECT_BY_POS) ) continue;

    if (OrderMagicNumber() == MagicNumber && OrderSymbol() == symbol && OrderCloseTime() == 0) {

      while (!IsTradeAllowed()) Sleep(500); // wait for Trade to be allowed(free context)
      switch (OrderType()) {
        // close open trades
        case OP_BUY:  result = _OrderClose(OrderTicket(),OrderLots(),Bid,5*pip,Red); break;
        case OP_SELL: result = _OrderClose(OrderTicket(),OrderLots(),Ask,5*pip,Red); break;
        // delete pending orders
        default:      result = _OrderDelete(OrderTicket(),CLR_NONE); break;
      }
    }
  }
}

#define MAX_RETRIES 10

//+------------------------------------------------------------------+
int _OrderSend(string symbol, int type, double lots, double price, int slippage, double stoploss, double takeprofit, string comment, int magic, datetime expiration, color arrow_color)
//+------------------------------------------------------------------+
{
//"reliable" version of OrderSend(): will retry 10 times before failing
  int i,j,err,ticket;
  
  for (i=0;i<MAX_RETRIES;i++) {
    for (j=0;j<MAX_RETRIES && !IsTradeAllowed();j++) Sleep(500); // wait for Trade to be allowed(free context)
    if (j==MAX_RETRIES) {
      _Alert("OrderSend() FAILED: trade not allowed");
      return (-1);
    }

    // send order with 0 SL, then modify order to apply SL -- a common counter-ECN method found in SteveHopwood's bots...
    ticket = OrderSend(symbol, type, lots, price, slippage, 0/*stoploss*/, takeprofit, comment, magic, expiration, arrow_color);
    if (ticket != -1) {
      if (stoploss!=0) _OrderModify(ticket, price, stoploss, takeprofit, expiration, arrow_color);
      return(ticket);//success
    }
  }

  err = GetLastError();
  _Alert("OrderSend() FAILED at "+price+" ("+err+"): "+ErrorDescription(err));
  return(-1);
}

//+------------------------------------------------------------------+
bool _OrderModify(int ticket, double price, double stoploss, double takeprofit, datetime expiration, color arrow_color) 
//+------------------------------------------------------------------+
{
//"reliable" version of OrderModify(): will retry 10 times before failing
  int i,j,err,ret;
  
  for (i=0;i<MAX_RETRIES;i++) {
    for (j=0;j<MAX_RETRIES && !IsTradeAllowed();j++) Sleep(500); // wait for Trade to be allowed(free context)
    if (j==MAX_RETRIES) {
      _Alert("OrderModify(#"+ticket+") FAILED: trade not allowed");
      return(false);
    }

    ret = OrderModify(ticket, price, stoploss, takeprofit, expiration, arrow_color); 
    if (ret == true) return(true);//success
  }

  err = GetLastError();
  _Alert("OrderModify(#"+ticket+") FAILED ("+err+"): "+ErrorDescription(err));
  return(false);
}

//+------------------------------------------------------------------+
bool _OrderClose( int ticket, double lots, double price, int slippage, color Color) 
//+------------------------------------------------------------------+
{
//"reliable" version of OrderClose(): will retry 10 times before failing
  int i,j,err;
  bool ret;
  
  for (i=0;i<MAX_RETRIES;i++) {
    for (j=0;j<MAX_RETRIES && !IsTradeAllowed();j++) Sleep(500); // wait for Trade to be allowed(free context)
    if (j==MAX_RETRIES) {
      _Alert("OrderClose(#"+ticket+") FAILED: trade not allowed");
      return(false);
    }

    ret = OrderClose(ticket, lots, price, slippage, Color);
    if (ret == true) return(true);//success
  }

  err = GetLastError();
  _Alert("OrderClose(#"+ticket+") FAILED ("+err+"): "+ErrorDescription(err));
  return(false);
}

//+------------------------------------------------------------------+
bool _OrderDelete(int ticket, color Color) 
//+------------------------------------------------------------------+
{
//"reliable" version of OrderDelete(): will retry 10 times before failing
  int i,j,err;
  bool ret;
  
  for (i=0;i<MAX_RETRIES;i++) {
    for (j=0;j<MAX_RETRIES && !IsTradeAllowed();j++) Sleep(500); // wait for Trade to be allowed(free context)
    if (j==MAX_RETRIES) {
      _Alert("OrderDelete(#"+ticket+") FAILED: trade not allowed");
      return(false);
    }

    ret = OrderDelete(ticket, Color);
    if (ret == true) return(true);//success
  }

  err = GetLastError();
  _Alert("OrderDelete(#"+ticket+") FAILED ("+err+"): "+ErrorDescription(err));
  return(false);
}


//+------------------------------------------------------------------+
int create_MagicNumber(string s)
//+------------------------------------------------------------------+
{
  // create a magic number that is "unique" for a given {EA_name,Symbol,Period} combo
  int magic=0;
  s = s+WindowExpertName()+Symbol()+Period();
  magic = hash_string(s);
  while (magic < 10000) {// magic number is not long enough, make another one
    s = s+magic;
    magic = hash_string(s);
  }
  return (magic);
}

//+------------------------------------------------------------------+
int hash_string(string s)
//+------------------------------------------------------------------+
{
  // this is the djb2 string hash algo
  int h = 5381, l = StringLen(s), i;
  for (i=0; i<l; i++) h = h * 33 + StringGetChar(s,i);
  return (h);
} /*hash_string()*/


//+------------------------------------------------------------------+
void getpip()
//+------------------------------------------------------------------+
{
   if(Digits==2 || Digits==4) pip = Point;
   else if(Digits==3 || Digits==5) pip = 10*Point;
   else if(Digits==6) pip = 100*Point;
      
	if (Digits == 3 || Digits == 2) digits = 2;
	else digits = 4;
	
	minlot = MarketInfo(Symbol(),MODE_MINLOT);
	
} /* getpip*/


//--------------------------------------------------------------------------------------
int time_string_to_int(string ts)
//--------------------------------------------------------------------------------------
// convert a time string "hh:mm" into an integer hhmm
{
  int t;
  t = StrToTime("1970.01.01 " + ts)/60;
  return (MathRound(t/60)*100+(t%60));
}

//--------------------------------------------------------------------------------------
string time_int_to_string(int t)
//--------------------------------------------------------------------------------------
// convert a time integer hhmm into a string "hh:mm"
{
  if (t>9999) return (StringSubstr(TimeToStr(t),11,5)); // time value includes DATE -- compatible with Indicator < V9
  return ((t/100)+":"+(t%100)); // time is in hhmm integer format (e.g 1400 for "14:00")
}

//--------------------------------------------------------------------------------------
int roundup_hhmm_time_to_TF (int hhmm, int tf)
//--------------------------------------------------------------------------------------
{
  double t  = hhmm; // convert to double for divisions
  double hh = MathFloor(t/100);
  int mm = (t/100-hh)*100;
  mm = (MathFloor(mm/tf)+1)*tf;
  return ( (hh+MathFloor(mm/60))*100 + mm-60*MathFloor(mm/60) );
}



//--------------------------------------------------------------------------------------
void _Alert(string text)
//--------------------------------------------------------------------------------------
{
  Alert ("LB EA: ", symbol, " ", text);
}

//+------------------------------------------------------------------+
bool import_LB_Indi_SETTINGS(string objPrefix, string symbol)
//+------------------------------------------------------------------+
// returns false if settings are missing
{
  bool ret=true;

  if (!GlobalVariableCheck(objPrefix+symbol+"-StartTime")) return(false);
  _iStartTime       = GlobalVariableGet(objPrefix+symbol+"-StartTime");

  if (!GlobalVariableCheck(objPrefix+symbol+"-EndTime")) return(false);
  _iEndTime         = GlobalVariableGet(objPrefix+symbol+"-EndTime");

  if (!GlobalVariableCheck(objPrefix+symbol+"-SessionEndTime")) return(false);
  _iSessionEndTime  = GlobalVariableGet(objPrefix+symbol+"-SessionEndTime");

  if (!GlobalVariableCheck(objPrefix+symbol+"-MaxBoxSizeInPips")) return(false);
  _MaxBoxSizeInPips = GlobalVariableGet(objPrefix+symbol+"-MaxBoxSizeInPips");

  if (!GlobalVariableCheck(objPrefix+symbol+"-EntryFactor")) return(false);
  _EntryFactor      = GlobalVariableGet(objPrefix+symbol+"-EntryFactor");

  if (!GlobalVariableCheck(objPrefix+symbol+"-TPFactor")) return(false);
  _TPFactor        = GlobalVariableGet(objPrefix+symbol+"-TPFactor");

  if (!GlobalVariableCheck(objPrefix+symbol+"-LevelsResizeFactor")) return(false);
  _LevelsResizeFactor = GlobalVariableGet(objPrefix+symbol+"-LevelsResizeFactor");

  if (!GlobalVariableCheck(objPrefix+symbol+"-MinExtentInPips")) return(false);
  _MinExtentInPips = GlobalVariableGet(objPrefix+symbol+"-MinExtentInPips");

  if (!GlobalVariableCheck(objPrefix+symbol+"-MaxExtentInPips")) return(false);
  _MaxExtentInPips = GlobalVariableGet(objPrefix+symbol+"-MaxExtentInPips");

  return (ret);
}

//+------------------------------------------------------------------+
bool update_local_LB_Indi_SETTINGS()
//+------------------------------------------------------------------+
// returns true if settings have changed
{
  bool ret=false;

  if (_iStartTime != iStartTime) {
    iStartTime = _iStartTime;
    StartTime  = time_int_to_string(iStartTime);
    ret=true;
  }
  if (_iEndTime != iEndTime) {
    iEndTime = _iEndTime;
    EndTime  = time_int_to_string(iEndTime);
    ret=true;
  }
  if (_iSessionEndTime != iSessionEndTime) {
    iSessionEndTime = _iSessionEndTime;
    SessionEndTime  = time_int_to_string(iSessionEndTime);
    ret=true;
  }
  if (_MaxBoxSizeInPips != 0 && _MaxBoxSizeInPips != MaxBoxSizeInPips) {
    MaxBoxSizeInPips = _MaxBoxSizeInPips;
    ret=true;
  }
  if (_EntryFactor != 0 && _EntryFactor != EntryFactor) {
    EntryFactor = _EntryFactor;
    ret=true;
  }
  if (_TPFactor != 0 && _TPFactor != TPFactor) {
    TPFactor = _TPFactor;
    ret=true;
  }
  if (_LevelsResizeFactor != 0 && _LevelsResizeFactor != LevelsResizeFactor) {
    LevelsResizeFactor = _LevelsResizeFactor;
    ret=true;
  }
  if (_MinExtentInPips != 0 && _MinExtentInPips != MinExtentInPips) {
    MinExtentInPips = _MinExtentInPips;
    ret=true;
  }
  if (_MaxExtentInPips != 0 && _MaxExtentInPips != MaxExtentInPips) {
    MaxExtentInPips = _MaxExtentInPips;
    ret=true;
  }

  return (ret);
}

//+------------------------------------------------------------------+
void compute_LB_Indi_LEVELS(datetime thistime)
//+------------------------------------------------------------------+
{
    tBoxStart = StrToTime(TimeToStr(thistime,TIME_DATE) + " "  + StartTime);
    tBoxEnd   = StrToTime(TimeToStr(thistime,TIME_DATE) + " "  + EndTime);
    if (tBoxStart > tBoxEnd) tBoxStart = tBoxStart - 24*60*60; // midnight wrap fix
    if (thistime < tBoxEnd) { // consider the last PAST box
      tBoxStart = tBoxStart - 24*60*60;
      tBoxEnd   = tBoxEnd   - 24*60*60;
    }

    tSessionStart = tBoxEnd;
    tSessionEnd = StrToTime(TimeToStr(tSessionStart,TIME_DATE) + " "  + SessionEndTime);
    if (tSessionStart > tSessionEnd) tSessionEnd = tSessionEnd + 24*60*60; // midnight wrap fix

    //get hi/lo
    StartShift = iBarShift(NULL,0,tBoxStart);
    EndShift   = iBarShift(NULL,0,tBoxEnd);
    boxHigh = High[iHighest(NULL,0,MODE_HIGH,(StartShift-EndShift+1),EndShift)];
    boxLow  = Low[iLowest(NULL,0,MODE_LOW,(StartShift-EndShift+1),EndShift)];
    boxExtent       = boxHigh - boxLow;
    boxMedianPrice = (boxHigh+boxLow)/2;

    // "resize" the Box with LevelsResizeFactor before computing levels
    double extent = boxExtent;
    if (MaxExtentInPips > 0 && boxExtent > MaxExtentInPips*pip) extent = MaxExtentInPips*pip;
    if (MinExtentInPips > 0 && boxExtent < MinExtentInPips*pip) extent = MinExtentInPips*pip;
    extent = extent * LevelsResizeFactor;
    vSellStop = boxMedianPrice + extent/2;
    vBuyStop  = boxMedianPrice - extent/2;

    // Entry and TP prices
    LongPendingEntryPrice  = NormalizeDouble(vSellStop + extent*EntryFactor, Digits);
    ShortPendingEntryPrice = NormalizeDouble(vBuyStop  - extent*EntryFactor, Digits);
    LongTP = NormalizeDouble(vSellStop + extent*TPFactor, Digits);
    ShortTP = NormalizeDouble(vBuyStop  - extent*TPFactor, Digits);
    LongSL = vBuyStop;
    ShortSL = vSellStop;
    StopLoss = (LongPendingEntryPrice-LongSL)/pip;
}

/*
void test()
{
  if (import_LB_Indi_SETTINGS(objPrefix,symbol) == true) {
  }

  // compute LEVELS values:
  compute_LB_Indi_LEVELS(Time[0]);

  
  // import_LB_Indi_LEVELS -- available since "London BreakoutV8.mq4" indicator
  Print(LongPendingEntryPrice+" = "+GlobalVariableGet(objPrefix+symbol+"-LongEntry"));
  Print(ShortPendingEntryPrice+" = "+GlobalVariableGet(objPrefix+symbol+"-ShortEntry"));
  Print(LongTP+" = "+GlobalVariableGet(objPrefix+symbol+"-LongTP"));
  Print(ShortTP+" = "+GlobalVariableGet(objPrefix+symbol+"-ShortTP"));
  Print(LongSL+" = "+GlobalVariableGet(objPrefix+symbol+"-LongSL"));
  Print(ShortSL+" = "+GlobalVariableGet(objPrefix+symbol+"-ShortSL"));

}
*/
//end

